@@ -168,9 +168,11 @@ class Model:
         return nstates_models + len(self._states)
 
     def nsignals(self):
+        """
+        :return: The number of signals.
+        """
         assert not self._under_construction, "Under construction."
-        totallist = [1,
-                     len(self._states) * 2,
+        totallist = [len(self._states) * 2,
                      len(self._inputs),
                      len(self._vars)
                      ] + [getattr(self, m).nsignals() for m in self._models]
@@ -182,6 +184,11 @@ class Model:
         return self._fmap_signals(lambda s: prefix + s,
                                   lambda m: getattr(self, m).signal_names(prefix + m + '.'))
 
+    def state_names(self, prefix=''):
+        assert not self._under_construction, "Under construction."
+        return self._fmap_states(lambda s: prefix + s,
+                                 lambda m: getattr(self, m).state_names(prefix + m + '.'))
+
     def state_vector(self):
         """
         Creates a flat state from the internal state and models
@@ -390,11 +397,17 @@ class Model:
             res.append(f_signal(v))
 
         res_models = [f_models(m) for m in self._models]
-        total = [np.array(res)] + res_models
-        total_flat = reduce(lambda a, b: np.concatenate(a, b), total)
+        assert res is not None, res
+        assert res_models is not None, res_models
+        total = [res] + res_models
+        assert total is not None
+
+        total_flat = reduce(lambda a, b: a + b, total)
+
+        assert total_flat is not None
 
         num = self.nsignals()
-        assert len(total_flat) == num, "Flatten worked."
+        assert len(total_flat) == num, f"Flatten failed: expected {num} results. Got {len(total_flat)} instead."
         return total_flat
 
     def __setattr__(self, key, value):
